-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

newTalent{
	name = "Water Strike",
	type = {"technique/arcane-water", 1},
	require = techs_req1,
	points = 5,
	random_ego = "attack",
	cooldown = 8,
	stamina = 12,
	tactical = { ATTACK = { weapon = 2 }, DISABLE = { stun = 2 } },
	action = function(self, t)
		--figure out which weapon (or none) to use
		local weapon = nil
		local offweapon = nil
		if self:getInven("MAINHAND") then
			weapon = self:getInven("MAINHAND")[1]
			if not weapon or not weapon.combat then
				weapon = nil
			end
		end
		if self:getInven("OFFHAND") then
			offweapon = self:getInven("OFFHAND")[1]
			if not offweapon or not offweapon.combat then
				offweapon = nil
			end
			if weapon == nil then
				weapon = offweapon
				offweapon = nil
			end
		end
		if self:attr("disarmed") or not weapon then
			weapon = self.combat
			offweapon = nil
		end

		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y then return nil end
		if target then
			if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end

			--If dual wielding, attack with second weapon first, so as to not affect the ice block
			if offweapon then
				local speed, hit = self:attackTargetWith(target, offweapon, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			end

			local speed, hit = self:attackTargetWith(target, weapon, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			-- Try to stun !
			if hit then
				if target:canBe("stun") then
					target:setEffect(target.EFF_FROZEN, 2, {hp=70 + self:combatPhysicalpower() * self:getTalentLevel(t), apply_power=self:combatPhysicalpower(), min_dur=1})
					game.flyers:add(x, y, 30, (rng.range(0,2)-1) * 0.5, -3, "Frozen!", {0,255,155})
				else
					game.flyers:add(x, y, 30, (rng.range(0,2)-1) * 0.5, -3, "Resist!", {0,255,155})
					game.logSeen(target, "%s resists!", target.name:capitalize())
				end
			end
		end
		--Even if we miss or no target, the power must explode in a ball!
		if self:getTalentLevel(t) >= 4 then
			local dur = 2
			local chance = math.floor(self:getTalentLevel(t) * 5)
			local hp = 70 + self:combatPhysicalpower() * self:getTalentLevel(t)
			self:project({type="ball", radius=math.floor(self:getTalentLevel(t)-3), selffire=false, friendlyfire=false}, x, y, DamageType.WATER_STRIKE_BALL, {dur=dur,chance=chance,hp=hp})
		end
		if not self:isTalentActive(self.T_WATER_STANCE) then self:forceUseTalent(self.T_WATER_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true}) end
		return true
	end,
	--Learn and unlearn effects, add/remove water stance
	--Increase/Decrease spell/water mastery by 0.03
	on_learn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 1 then
			self:learnTalent(self.T_WATER_STANCE, true, nil, {no_unlearn=true})
		end
		self:setTalentTypeMastery("spell/water", self:getTalentTypeMastery("spell/water") + 0.03)
	end,
	on_unlearn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 0 then
			self:unlearnTalent(self.T_WATER_STANCE)
		end
		self:setTalentTypeMastery("spell/water", self:getTalentTypeMastery("spell/water") - 0.03)
	end,

	info = function(self, t)
		local radius = math.max(math.floor(self:getTalentLevel(t)-3),0)
		local chance = math.floor(self:getTalentLevel(t) * 5)
		return ([[Strike with the power of Water. Freezes target for 2 turns and deals %d%% damage.  Chance to freeze increases with Physical Power.
		At level 4 it blasts a ball of water of radius %d which has a %d%% chance of freezing enemies as well.  After use, you change to the Water Stance.]]):
		format(100 * self:combatTalentWeaponDamage(t, 1, 1.5),radius,chance)
	end,
}

newTalent{
	name = "Healing Waters",
	type = {"technique/arcane-water", 2},
	require = techs_req2,
	points = 5,
	mana = 25,
	cooldown = 16,
	tactical = { HEAL = 2 },
	getHeal = function(self, t) return 40 + self:combatTalentSpellDamage(t, 10, 520) end,
	is_heal = true,
	is_spell = true,
	action = function(self, t)
		self:heal(self:spellCrit(t.getHeal(self, t)), self)
		game:playSoundNear(self, "talents/heal")
		return true
	end,
	--Increase/Decrease spell/water mastery by 0.03
	on_learn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		self:setTalentTypeMastery("spell/water", self:getTalentTypeMastery("spell/water") + 0.03)
	end,
	on_unlearn = function(self, t)
		self:setTalentTypeMastery("spell/water", self:getTalentTypeMastery("spell/water") - 0.03)
	end,
	info = function(self, t)
		local heal = t.getHeal(self, t)
		return ([[Healing waters flow over you, healing your body for %d life.
		The life healed will increase with your Spellpower.]]):
		format(heal)
	end,
}

newTalent{
	name = "Tidal Rush",
	type = {"technique/arcane-water", 3},
	require = techs_req3,
	points = 5,
	random_ego = "attack",
	cooldown = 20,
	stamina = 25,
	range = function(self,t) return math.min(3 + self:getTalentLevelRaw(t),10) end,
	tactical = { CLOSEIN = 2 },
	requires_target = true,
	waveTarget = function(self, t)
		return {type="ball", range=0, radius=1, selffire=false, talent=t}
	end,
	getWaveDamage = function(self, t) return self:combatTalentSpellDamage(t, 10, 100) end,
	action = function(self, t)
		local targeting = {type="bolt", range=self:getTalentRange(t), nolock=true}
		local targetX, targetY, actualTarget = self:getTarget(targeting)
		--debug
		if not targetX or not targetY then return nil end
		if core.fov.distance(self.x, self.y, targetX, targetY) > self:getTalentRange(t) then return nil end
		if core.fov.distance(self.x, self.y, targetX, targetY) < 2 then
			game.logSeen(self, "Tidal Rush's target is too close!  It must be at least 2 spaces away.")
			return nil
		end

		local block_actor = function(_, bx, by) return game.level.map:checkEntity(bx, by, Map.TERRAIN, "block_move", target) end
		local lineFunction = core.fov.line(self.x, self.y, targetX, targetY, block_actor)
		local nextX, nextY, is_corner_blocked = lineFunction:step()
		local moved = false
		--Ok, start to loop (emergency break conditions included if we somehow run out of next's or blocked)
		while nextX and nextY and not is_corner_blocked do

			--First we mini-wave against all targets in a ball of radius 1
			local tgwave = t.waveTarget(self,t)
			self:project(tgwave, self.x, self.y, DamageType.PHYSKNOCKBACK, {dam=self:spellCrit(t.getWaveDamage(self, t)),x=self.x,y=self.y,dist=1})
			game.level.map:particleEmitter(self.x, self.y, tgwave.radius, "ball_ice", {radius=tgwave.radius})
			local blockingTarget = game.level.map(nextX, nextY, Map.ACTOR)
			moved = false
			--If someone in the way, swap positions with them - the undertow pulls them away!
				if blockingTarget then
					-- Displace
					local tx,ty,sx,sy = blockingTarget.x,blockingTarget.y, self.x, self.y
					self.x = nil self.y = nil
					self:move(tx, ty, true)
					blockingTarget.x = nil blockingTarget.y = nil
					blockingTarget:move(sx, sy, true)
					game.logSeen(self, "%s is pulled away by the water's undertow!", blockingTarget.name)
					moved = true
				end
			if not moved then
				-- check that we can move
				if not game.level.map:isBound(nextX, nextY) or game.level.map:checkAllEntities(nextX, nextY, "block_move", self) then break end

				--do the move
				self:move(nextX,nextY, true)
			end
			--if now adjacent to target, break, or else hand over the next position
			if core.fov.distance(self.x, self.y, targetX, targetY) == 1 then
				break
			else
				nextX, nextY, is_corner_blocked = lineFunction:step()
			end
		end
		--We are now presumably next to the target, or something has gone horribly wrong
		--Attack the final target
		if actualTarget then
			if core.fov.distance(self.x, self.y, actualTarget.x, actualTarget.y) <= 1 then
				self:attackTarget(actualTarget, nil, self:combatTalentWeaponDamage(t, 1, 1.5), true)
			end
		end
		game:playSoundNear(self, "talents/tidalwave")
		return true
	end,
	--Increase/Decrease spell/water mastery by 0.03
	on_learn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		self:setTalentTypeMastery("spell/water", self:getTalentTypeMastery("spell/water") + 0.03)
	end,
	on_unlearn = function(self, t)
		self:setTalentTypeMastery("spell/water", self:getTalentTypeMastery("spell/water") - 0.03)
	end,
	info = function(self, t)
		local dam = t.getWaveDamage(self,t)
		return ([[Summon a huge tide and flow along it, knocking back foes or having the undertow wash them away.
		The sheer force of the wave deals %0.2f physical damage.  At the end of the wave, attack your true target for %d%% extra damage.]]):format(dam,100 * self:combatTalentWeaponDamage(t, 1, 1.5))
	end,
}


newTalent{
	name = "Drain Water of Life",
	type = {"technique/arcane-water", 4},
	require = techs_req4,
	points = 5,
	random_ego = "attack",
	cooldown = 16,
	mana = 35,
	tactical = { ATTACK = { weapon = 2 }, HEAL = 2 },
	action = function(self, t)
		--figure out which weapon (or none) to use
		local weapon = nil
		local offweapon = nil
		if self:getInven("MAINHAND") then
			weapon = self:getInven("MAINHAND")[1]
			if not weapon or not weapon.combat then
				weapon = nil
			end
		end
		if self:getInven("OFFHAND") then
			offweapon = self:getInven("OFFHAND")[1]
			if not offweapon or not offweapon.combat then
				offweapon = nil
			end
			if weapon == nil then
				weapon = offweapon
				offweapon = nil
			end
		end
		if self:attr("disarmed") or not weapon then
			weapon = self.combat
			offweapon = nil
		end

		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y then return nil end
		if target then
			if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end

			self:attr("drain_water_of_life", 1)
			if offweapon then
				local speed, hit = self:attackTargetWith(target, offweapon, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			end

			local speed, hit = self:attackTargetWith(target, weapon, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			self:attr("drain_water_of_life", -1)
		end
		return true
	end,
	--Increase/Decrease spell/water mastery by 0.03
	on_learn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		self:setTalentTypeMastery("spell/water", self:getTalentTypeMastery("spell/water") + 0.03)
	end,
	on_unlearn = function(self, t)
		self:setTalentTypeMastery("spell/water", self:getTalentTypeMastery("spell/water") - 0.03)
	end,

	info = function(self, t)
		local radius = math.max(math.floor(self:getTalentLevel(t)-3),0)
		local chance = math.floor(self:getTalentLevel(t) * 5)
		return ([[Strike with your weapon and drain the waters of life from your foe, healing yourself.  Your weapon deals %d%% damage and you heal %d%% of the damage dealt.]]):
		format(100 * self:combatTalentWeaponDamage(t, 1, 1.5),self:getTalentLevelRaw(t)*20)
	end,
}


newTalent{
	name = "Water Stance",
	type = {"technique/elemental-stance", 1},
	points = 1,
	mode = "sustained",
	cooldown = 0,
	elemental_stance = true,
	getStanceHealMod = function(self,t) return (self:combatSpellpower() / 1000.0) * (1 +self:getTalentLevel(self.T_ARCANE_STANCE)) end,
	getStanceRegenMod = function(self,t) return (self:combatSpellpower() / 100.0) * (1 + self:getTalentLevel(self.T_ARCANE_STANCE)) end,
	on_pre_use = function(self, t, silent)
		-- prevent anyone from activating other than without the "force_talent_ignore_ressources" flag
		if not self:attr("force_talent_ignore_energy") then
			if not silent then game.logPlayer(self, "You can not active Water Stance directly.  Use Water Strike to activate it.") end
			return false
		end
		return true
	end,
	activate = function(self,t)
		local stance_turned_off = nil
		self:attr("water_stance", 1)
		if self:isTalentActive(self.T_AIR_STANCE) then
			self:forceUseTalent(self.T_AIR_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:isTalentActive(self.T_EARTH_STANCE) then
			self:forceUseTalent(self.T_EARTH_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:isTalentActive(self.T_FIRE_STANCE) then
			self:forceUseTalent(self.T_FIRE_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if not stance_turned_off and self:isTalentActive(self.T_ARCANE_DESTRUCTION) then
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		return {
			heal_inc = self:addTemporaryValue("healing_factor", t.getStanceHealMod(self,t)),
			regen_inc = self:addTemporaryValue("life_regen", t.getStanceRegenMod(self,t)),
		}
	end,
	deactivate = function(self,t,p)
		self:attr("water_stance", -1)
		--turn Arcane Destruction on and off if on.
		if self:isTalentActive(self.T_ARCANE_DESTRUCTION) then
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		self:removeTemporaryValue("healing_factor", p.heal_inc)
		self:removeTemporaryValue("life_regen", p.regen_inc)
		return true
	end,
	info = function(self, t)
		local healinc = t.getStanceHealMod(self, t)
		local regeninc = t.getStanceRegenMod(self, t)
		return ([[Enter the elemental stance of Water by using the Water Strike talent.  Water Stance increases your healing effects by %d%% and your regeneration by %0.2f.  Benefits increase with the Arcane Stance talent.]])
		:format(healinc * 100, regeninc)
	end,
}
